/**
 * @fileoverview the main of the experiment
 * 
 * @author Bo Carlson (bo@chairseven.com)
 */

/**
 * Experiment Main
 * 
 * Closure otherwise events freak out and this jumps to window
 * 
 * You now no longer need to push this up to global using $main
 * or $this while maintaining this. Refer to _Main for this.
 * Please do not send anything to global unless you absolutely have to.
 * You're safe in here.
 * 
 * Anything with a _ can be referenced inside anything inside this but will
 * not be seen from outside. Anything with a var cannot be seen outside of 
 * this. Anything put on this will be seen outside of this.
 * 
 * @param {object} arga (required) - the ARGA wrapper
 * @param {object} xmlParser (required) - The XML Parser
 * @param {jQuery} appContainer (required) - the app container
 * @param {boolean} debug (optional) - set to true for debug info
 * 
 * @constructor
 */
function Main(Arga,XMLParser,appContainer,logging,debug){
    //this
    var _Main = this,
	
    //give this objects
    _Arga               = Arga,
    _XMLParser          = XMLParser,
    _CommonAll          = new CommonAll,
    _CommonExperiment   = new CommonExperiment,
	
    //general
    _appContainer       = appContainer,
    _time               = 1000, //time in ms for application loop
    _minute             = 60000, //second in ms
    _currTime           = 0, //current application time
    _textLineWidth      = 900, //line width for makeTextBox
     tabindex           = platform.iOS ? ' tabindex="0" ' : '',
     experimentBox      = $('<div id="experimentBox"' + tabindex + '>Press the space bar' + (platform.mobile ? ' or <span class="visually-hidden">double</span> tap' : '') + '<br/> to begin the experiment</div>'),
    _boxClass           = 'textBox',
    _blockContainer     = $('<div id="block"/>' ).hide(),
    live                = platform.android || platform.firefox || platform.iOS ? 'assertive' : 'polite',
    role                = platform.android ? '' : 'role="log"',
    liveFeedback        = $('<div id="live-feedback" class="visually-hidden"' + role + ' aria-live="' + live + '"></div>'),
    $header             = $('<div id="header"' + tabindex + 'class="fullBar"/>'),
    _logging            = typeof logging == "boolean" ? logging : false,
    _debug              = typeof debug == "boolean" ? debug : false;
	
    /**
	 * !!!!START DEBUG CODE!!!!
	 * 
	 * You can load your Arga objects here to make sure the page is setting the values correctly
	 * please remove this when you are done.
	 * 
	 * Also please make sure to update the object in js/arga/arga_wrapper.js before turning your
	 * module in for review. The people hooking this up need this info.
	 */
    //22 lines = one round of logging
    var AMChartOutputLines = 0;
	
    if(_debug){
        //call the common debug method
        _CommonAll.debug();
		
        //All of the values for the advanced settings page
        _Arga.setAdvancedSettingsData({
            faces: 1, 
            words: 0
        });
		
        //All of the values for the experiment state
        _Arga.setExperimentData({
            answers:{}
        });
    }
    /**
    	 * !!!!END DEBUG CODE!!!!
    	 */		
		

    //experiment specific data
    /**
    	 * @todo: I think this might be better off as a cookie
    	 * some of the experiments want the user to be able to
    	 * start where they left off. Ideas?
    	 */
    var _experimentState    = _Arga.getExperimentData(),
    _experimentSettings = _Arga.getAdvancedSettingsData(),
    _experimentData     = _XMLParser.buildDataObject(),
    _theList            = [],
    _blockCount         = 0,
    _blockRound         = 0,
    _count              = 0,
    _answers            = [],
    _trials             = [],
    _appDate            = new Date(),
    _roundStart         = 0,
    _roundEnd           = 0,
    _appSystemTime      = 0,
    state               = 0;

    if (window.opener.player.EXP_FACES > -1) {
        _experimentSettings.faces = window.opener.player.EXP_FACES;
    }
    if (window.opener.player.EXP_WORDS > -1) {
        _experimentSettings.words = window.opener.player.EXP_WORDS;
    }

    //alert(JSON.stringify(_experimentSettings));
    
  var beginExpFb = platform.mobile ? 'Press the space bar or double tap to begin the experiment' : 'Press the space bar or tap to begin the experiment';
  var beginBlockStrFb = platform.mobile ? 'Press the space bar or double tap to begin the block' : 'Press the space bar or tap to begin the block';
   
   // Initialize persistent class data storage.
    _Arga.getExperimentClassData();
        
    //build the list of letters to show
    buildList();
		
    //Start it up
    buildHTML();
    startLogging();

    //log start if logging
    function startLogging(){
        if(_logging){
            _appSystemTime = new Date();
            _CommonAll.log("Experiment ready");
            _CommonExperiment.startExperimentTiming();
        }

      if (!platform.mobile) {
        setTimeout(function () {
          liveFeedback.text(beginExpFb);
        }, 2000);
      }

      if (platform.android) {
        setTimeout(function () {
          document.querySelector('#experiment-container').focus();
          liveFeedback.text(beginExpFb);
        }, 1000);
      }

      if (platform.iOS) {
        $('body').prepend($('<span id="focus_elem" tabindex="0">Experiment Tab</span>'));
        setTimeout(function () {
          for (var i = 0; i < 300; i++) {
            setTimeout(function () {
              $('#focus_elem').focus();
            }, 100);
          }
          liveFeedback.text(beginExpFb);
        }, 3000);
      }
    }
	
    //display the html
    function display(){
      if (state === 0) {
        _blockContainer.show();
        experimentBox.hide();
      }
        clearContent();
        if(_count < _theList.length){
            if(_theList[_count].display == "instructions"){
                if (state === 0 || state === 2) {
                  state = 1;
                }

                _blockCount++;
                _blockRound = 0;
                instructions(_theList[_count]);
            }
            else {
              if (state === 1) {
                state = 2;
              }
                $('#trialCounter').html('Trial <span id="currentTrial">' + (_count + 1 - _blockCount) + '</span> of <span id="totalTrials">' + (_theList.length - _experimentData.blocks.length) + '</span>');

                var trialStr = 'Trial ' + (_count + 1 - _blockCount) + ' of '
                  + (_theList.length - _experimentData.blocks.length);
                _blockRound++;
                blockRound(_theList[_count], trialStr);
            }
        }
        else{
            state = 3;
            finish();
        }
    }
	
    //display instructions
    function instructions(data){
        //put the info into the html containers
        $header.html(data.title);
        $('#leftChoice').html('<span>'+data.leftInput+'</span>');
        $('.append').html('<span>Ctrl-Z = '+data.leftInput.replace('<br/>','; ')+'</span><br/><span>Ctrl-/ = '+data.rightInput.replace('<br/>','; ')+'</span>');
        $('#rightChoice').html('<span>'+data.rightInput+'</span>');
        $('#instructions').html(data.instructions);

      if (platform.android) {
        document.querySelector('#experiment-container').focus();
      }

      if (platform.iOS) {
        $header.focus();
      }

      var instr1 = ' press z, to ';
      var instr2 = ' press slash. ';

      if (platform.mobile) {
        instr1 = ' press z or swipe to the left, to ';
        instr2 = ' press slash or swipe to the right. '
      }

      liveFeedback.text(data.title + '. You will be presented with a set of '
        + (data.stimulus === 'mix' ? 'words and images' : data.stimulus === 'faces' ? 'images' : 'words')
        + ' to classify into ' + data.leftInput.replace('<br/>',' or ') + instr1
        + data.rightInput.replace('<br/>',' or ') + instr2 + beginBlockStrFb);
    }
	
    //display a block round
    function blockRound(data, trialStr){
        if(typeof data.stimulus.img != "undefined"){
            var blockImg = $('<img class="append" tabindex="0" alt="image"/>');
			
            blockImg.load({
                data:data
            },function(e){
                //get the round start time
                _roundStart = new Date();
				
                //log the start of the round if logging
                if(_logging){
                    _CommonAll.log("Block: "+_blockCount+" Round: "+_blockRound+" START");
                    _CommonExperiment.eventStart("block"+_blockCount+"round"+_blockRound,_appDate.getTime()-_appSystemTime);
                }
				
                $('#leftChoice').html("<span>"+e.data.data.leftInput+"</span>");
                if(Boolean(data.stimulus.text)){
                    $('.append').text(data.stimulus.text);
                }
                $('#rightChoice').html("<span>"+e.data.data.rightInput+"</span>");
            });
			
            blockImg.attr("src",data.stimulus.img);
            $('#stimulus').append(blockImg);

            var feedbackStr = trialStr + ". image";

          liveFeedback.text("");
        }
        else{
            //get the round start time
            _roundStart = new Date();
			
            //log the start of the round if logging
            if(_logging){
                _CommonAll.log("Block: "+_blockCount+" Round: "+_blockRound+" START");
                _CommonExperiment.eventStart("block"+_blockCount+"round"+_blockRound,_appDate.getTime()-_appSystemTime);
            }
			
            //put the info into the html containers
            $('#leftChoice').html("<span>"+data.leftInput+"</span>");
            $('.append').text(data.stimulus.text);
            $('#rightChoice').html("<span>"+data.rightInput+"</span>");

            var feedbackStr = trialStr + ". " + data.stimulus.text;
        }

      if (!platform.mobile) {
        liveFeedback.text(feedbackStr);
      }
      if (platform.iOS) {
        setTimeout(function() {
          $('.append').focus();
          liveFeedback.html( feedbackStr );
        }, 500);
      }
      if (platform.android) {
        setTimeout(function() {
          $('.append').focus();
          liveFeedback.html(feedbackStr);
        }, 500);
      }
    }
	
    function finish(){
        _appContainer.empty();
        $("#experiment").append('<div class="experimentOver"> Thank you for your participation. <br /> The experimental procedure is over. </div>');
        $("#close").show();
        // brb: need to hide the experiment div so that the close button
        // isn't pushed to the bottom of the screen
        $("#experiment").css("display", "none");
        
        setTimeout(function() {
          $("#close button").focus();
        }, 1000);

        setTimeout(function() {
          liveFeedback.text('Thank you for your participation. The experimental procedure is over.');
        }, 3000);

        //send Arga the info
        _Arga.setExperimentData({
            answers: _answers, 
            trials: _trials
        });
		
        //if logging display complete message and Arga Data
        if(_logging){
            _CommonAll.log("Test Message");
            _CommonAll.log(JSON.stringify({
                answers: _answers, 
                trials: _trials
            }));
        //_CommonExperiment.dumpTestingTimelineData(AMChartOutputLines);
        }
    }
	
    function blockResponse(data,e){
      e.stopPropagation();
      e.preventDefault();
      var result = '';
      if (platform.mobile) {

        if (e.currentTarget.id === 'leftChoice') {
          result = 'Z';
        }
        else {
          result = '/';
        }
      }
      else {
        if (e.which == 90) {
          result = 'Z';
        }
        else if (e.which == 191) {
          result = '/';
        }
      }

            _roundEnd = (new Date())-_roundStart;
			
            //log the start of the round if logging
            if(_logging){
                _CommonAll.log("Block: "+_blockCount+" Round: "+_blockRound+" END "+_roundEnd+"ms");
                _CommonExperiment.eventFinish("block"+_blockCount+"round"+_blockRound,_appDate.getTime()-_appSystemTime);
            }
			
            if(data.grade){
                _answers.push({
                    response: result === 'Z' ? data.leftInput.replace('<br/>',' ') : data.rightInput.replace('<br/>',' '),
                    answer: {
                        face: data.stimulus.face, 
                        word: data.stimulus.text
                    }, 
                    reverse: data.reverse, 
                    time: _roundEnd
                });
            }
            
            var stimulus, response;
            
            // Figure out the stimulus for this block type.
            if (_theList[_count].stimulus.type == 'words') {
                stimulus = _theList[_count].stimulus.text;
            } else if (_theList[_count].stimulus.type == 'faces') {
                stimulus = _theList[_count].stimulus.face;
            } else {
                stimulus = _theList[_count].stimulus.text + ';' + _theList[_count].stimulus.face;
            }
            
            // Figure out their response.
            response = (result === 'Z' ? data.leftInput.replace('<br/>',';') : data.rightInput.replace('<br/>',';'));
            
            // Expanded trial logging.
            _trials.push({
                trial: _count,
                block: _theList[_count].stimulus.type,
                reverse: data.reverse,
                stimulus: stimulus.toLowerCase(),
                response: response.toLowerCase(),
                responseTime: _roundEnd
            });
            
            _count++;
            display();
        }
	
    function buildHTML(){
        var $leftElem, $rightElem;

        if (platform.mobile) {
          $leftElem = $('<div class="side"><button id="leftChoice"/></div>');
          $rightElem = $('<div class="side"><button id="rightChoice"/></div>');
        }
        else {
          $leftElem = $('<div class="side" id="leftChoice"/>');
          $rightElem = $('<div class="side" id="rightChoice"/>');
        }
 
        _blockContainer.append($header);
        _blockContainer.append($leftElem);
        _blockContainer.append($('<div id="stimulus" class="middle"><span class="append" tabindex="0"></span></div>'));
        _blockContainer.append($rightElem);
        _blockContainer.append($('<div id="instructions" class="fullBar"/>'));
        _blockContainer.append($('<div id="trialCounter" ></div>'));

        _appContainer.append(_blockContainer);
        _appContainer.append(experimentBox);

      //init live feedback
      $('#experiment-container').prepend(liveFeedback);

      var event = platform.iOS ? 'touchend' : 'click';

      if (platform.mobile) {
        $(document).on(event, function(e) {
          e.stopPropagation();
          e.preventDefault();
          switch (state) {
            case 1:
              //display next block
              _count++;
            case 0:
              //hide instruction, display 1st block
              display();
              break;
            default:
              break;
          }
        });
        $('#leftChoice, #rightChoice').on(event, function(e) {
          e.stopPropagation();
          e.preventDefault();
          switch (state) {
            case 2:
              blockResponse(_theList[_count], e);
              break;
            default:
              break;
          }
        });
      }
      else {
        $(document).keydown(function(e){
          //space pressed
          if (e.which == 32) {
            switch (state) {
              case 1:
                //display next block
                e.preventDefault();
                _count++;
              case 0:
                //hide instruction, display 1st block
                display();
                break;
              default:
                break;
            }
          }
          //Z or slash pressed
          if (e.which == 90 || e.which == 191) {

            // brb: requiring ctrl key pressed in conjuction with 'Z' or '/'

            if (!e.ctrlKey) {
              console.log('Ctrl key not pressed');
              return;
            }

            switch (state) {
              case 2:
                blockResponse(_theList[_count], e);
                break;
              default:
                break;
            }
          }
        });
      }

      $('.close-button button').on(event, function() {
        try {
          window.opener.player.close_experiment();
        } catch(e) {

        }
      });
    }
	
    /**
    	 * Clears the content inside of the HTML
    	 * Does not remove HTML from the screen
    	 */
    function clearContent(){
        $('.fullBar').empty();
        $('#leftChoice').empty();
        $('#rightChoice').empty();
        $('.append').empty();
        $('#stimulus img').remove();
    }

    /**
    	 * Build the list of words
    	 * 
    	 * This is here to demo what you can do with data in the XML file
    	 * feel free to modify it or remove it all together if you need to
    	 * 
    	 * @return
    	 */
    function buildList(){
        var maxBlocks  = parseInt(_experimentData.blocks.length),
        wordTotal  = parseInt(_experimentData.wordList[_experimentSettings.words].total),
        dspWords   = objToArray(_experimentData.words[_experimentSettings.words]),
        wordType   = _experimentSettings.words,
        dspFaces   = objToArray(_experimentData.faces[_experimentSettings.faces].choices),
        faceType   = _experimentSettings.faces,
        picTotal   = parseInt(_experimentData.faces[_experimentSettings.faces].picCount),
        rounds     = 0,
        revOrder   = _CommonAll.randomIndex(2),
        listItem   = function(display,reverse,leftInput,rightInput,stimulus,title,instructions,grade) {
            // Uppercase the category if a combined trial.
            if (leftInput && leftInput.split('<br/>').length > 1) {
                leftInput = leftInput.split('<br/>')[0].toUpperCase() + '<br/>' + leftInput.split('<br/>')[1];
            } else if (leftInput.length <= 4) {
                // Otherwise, only uppercase the "bad/good" options.
                leftInput = leftInput.toUpperCase();
            }
                            
            // Same thing for the right side.
            if (rightInput && rightInput.split('<br/>').length > 1) {
                rightInput = rightInput.split('<br/>')[0].toUpperCase() + '<br/>' + rightInput.split('<br/>')[1];
            } else if (rightInput.length <= 4) {
                rightInput = rightInput.toUpperCase();
            }
                
            return {
                display: Boolean(display) ? display : null,
                reverse: reverse,
                leftInput: Boolean(leftInput) ? leftInput : null,
                rightInput: Boolean(rightInput) ? rightInput : null,
                stimulus: Boolean(stimulus) ? stimulus : null,
                title: Boolean(title) ? title : null,
                instructions: Boolean(instructions) ? instructions : null,
                grade: Boolean(grade) ? true : false
            };
        },
        usedPics   = [],
        tmpItem    = null,
        gradeBlock = false,
        balancer   = {},
        notFound   = true;
						 
        //reverse the inputs so each student won't get the same input
        if(Boolean(revOrder)){
            dspWords.reverse();
            dspFaces.reverse();
        }
						 
        //loop the blocks
        for(var i = 0; i < maxBlocks; i++){						
            if(_experimentData.blocks[i].type == "faces"){
                var reverse  = Boolean(parseInt(_experimentData.blocks[i].reverse)),
                    maxFaces = parseInt(_experimentData.faces[_experimentSettings.faces].picCount);

                //reverse the inputs if needed
                if(reverse){
                    dspFaces.reverse();
                }
				
                //build instructions for the block

              var instruct = '';
              if (_experimentData.blocks[i].instructions) {
                instruct = 'Press the space bar' + (platform.mobile ? ' or <span class="visually-hidden">double</span> tap' : '') + ' to begin the block'
              }

                tmpItem = new listItem("instructions",
                    reverse,
                    dspFaces[0],
                    dspFaces[1],
                    "faces",
                    _experimentData.blocks[i].title,
                  instruct);
				
                _theList.push(tmpItem);
				
                rounds = parseInt(_experimentData.faces[_experimentSettings.faces].rounds);
	
                //build the balancer
                balancer = {};
                balancer[dspFaces[0]] = [];
                balancer[dspFaces[1]] = [];
				
                //now do the rounds
                for(var j = 0; j < rounds; j++){
                    notFound = true;
					
                    while(notFound){						
                        var faceNum  = _CommonAll.randomIndex(2), //get a random face type
                        pic      = _CommonAll.randomIndex(maxFaces); //and a random pic

                        if($.inArray(pic, balancer[dspFaces[faceNum]]) === -1 && balancer[dspFaces[faceNum]].length < maxFaces){
                            balancer[dspFaces[faceNum]].push(pic);
							
                            _theList.push(new listItem("block",
                                reverse,
                                dspFaces[0],
                                dspFaces[1],
                                {
                                    face: dspFaces[faceNum],
                                    img: "iaimages/folder"+_experimentSettings.faces+"/"+dspFaces[faceNum]+"/"+pic+".jpg",
                                    type: _experimentData.blocks[i].type
                                })
                            );
                            
                            notFound = false;
                        }
                    }
                }
				
                //set them back to normal for the next round
                if(reverse){
                    dspFaces.reverse();
                }
            }
            else if(_experimentData.blocks[i].type == "words"){
                var reverse  = Boolean(parseInt(_experimentData.blocks[i].reverse));
				
                //reverse the inputs if needed
                if(reverse){
                    dspWords.reverse();
                }
				
                //build instructions for the block

              var instruct = '';
              if (_experimentData.blocks[i].instructions) {
                instruct = 'Press the space bar' + (platform.mobile ? ' or <span class="visually-hidden">double</span> tap' : '') + ' to begin the block'
              }

                tmpItem = new listItem("instructions",
                    reverse,
                    dspWords[0],
                    dspWords[1],
                    "words",
                    _experimentData.blocks[i].title,
                  instruct);
				
                _theList.push(tmpItem);

                rounds = parseInt(_experimentData.wordList[_experimentSettings.words].total);

                //build the balancer
                balancer = {};
                balancer[(dspWords[0].toLowerCase()).replace(" ","_")] = [];
                balancer[(dspWords[1].toLowerCase()).replace(" ","_")] = [];
				
                //now do the rounds
                for(var j = 0; j < rounds; j++){
                    notFound = true;
					
                    while(notFound){
                        var wordList = _experimentSettings.words,
                        wordNum  = _CommonAll.randomIndex(2), //get a random word type
                        wordType = (dspWords[wordNum].toLowerCase()).replace(" ","_"), //type of words from the list
                        listLen  = parseInt(_experimentData.wordList[wordList][wordType].length), //length of word type selected
                        word     = _CommonAll.randomIndex(listLen); //and a random word
						
                        if($.inArray(word, balancer[wordType]) === -1 && balancer[wordType].length < listLen){
                            tmpItem = new listItem("block",
                                reverse,
                                dspWords[0],
                                dspWords[1],
                                {
                                    text: _experimentData.wordList[wordList][wordType][word],
                                    type: _experimentData.blocks[i].type
                                });
							
                            _theList.push(tmpItem);
							
                            balancer[wordType].push(word);
							
                            notFound = false;
                        }
                    }
                }
				
                //set them back to normal for the next round
                if(reverse){
                    dspWords.reverse();
                }
            }
            else{
                var list1Max = 0,
                list2Max = 0,
                maxTotal = null,
                reverse  = Boolean(parseInt(_experimentData.blocks[i].reverse));
					
                //reverse the inputs if needed				
                if(reverse){
                    dspWords.reverse();
                }
				
                //build instructions for the block
              var instruct = '';
              if (_experimentData.blocks[i].instructions) {
                instruct = 'Press the space bar' + (platform.mobile ? ' or <span class="visually-hidden">double</span> tap' : '') + ' to begin the block'
              }
                tmpItem = new listItem("instructions",
                    reverse,
                    dspWords[0]+"<br/>"+dspFaces[0],
                    dspWords[1]+"<br/>"+dspFaces[1],
                    "mix",
                    _experimentData.blocks[i].title,
                  instruct);
				
                _theList.push(tmpItem);
				
                //figure out the length of choice 0
                if(parseInt(_experimentData.wordList[_experimentSettings.words][dspWords[0].toLowerCase().replace(' ', '_')].length) > 
                    parseInt(_experimentData.faces[_experimentSettings.faces].picCount)){
                    list1Max = parseInt(_experimentData.faces[_experimentSettings.faces].picCount);
                }
                else{
                    list1Max = parseInt(_experimentData.wordList[_experimentSettings.words][dspWords[0].toLowerCase().replace(' ', '_')].length);
                }
                //figure out the length of choice 1
                if(parseInt(_experimentData.wordList[_experimentSettings.words][dspWords[1].toLowerCase().replace(' ', '_')].length) > 
                    parseInt(_experimentData.faces[_experimentSettings.faces].picCount)){
                    list2Max = parseInt(_experimentData.faces[_experimentSettings.faces].picCount);
                }
                else{
                    list2Max = parseInt(_experimentData.wordList[_experimentSettings.words][dspWords[1].toLowerCase().replace(' ', '_')].length);
                }
				
                rounds = list1Max+list2Max;
                maxTotal = [list1Max,list2Max];
				
                //build the balancer
                balancer = {};
                balancer[(dspWords[0].toLowerCase()).replace(" ","_")] = [];
                balancer[(dspWords[1].toLowerCase()).replace(" ","_")] = [];
				
                balancer[dspFaces[0]] = [];
                balancer[dspFaces[1]] = [];
				
                //now do the rounds				
                for(var j = 0; j < rounds; j++){
                    var tmpWord = "",
                    tmpFace = "";
                    
                    if ( ( Math.floor(Math.random() * (2)) ) == 1 ) {
                        var whichDisplay = "first";
                    } else {
                        var whichDisplay = "second";
                    }
                    
                    if(whichDisplay == "first"){		
                        notFound = true;
					
                        //find the word
                        while(notFound){
                            var wordList = _experimentSettings.words,
                            wordNum  = _CommonAll.randomIndex(2), //get a random word type
                            wordType = (dspWords[wordNum].toLowerCase()).replace(" ","_"), //type of words from the list
                            listLen  = parseInt(_experimentData.wordList[wordList][wordType].length), //length of word type selected
                            word     = _CommonAll.randomIndex(listLen); //and a random word
					
                            if($.inArray(word, balancer[wordType]) === -1 && balancer[wordType].length < listLen){
                                tmpWord = _experimentData.wordList[wordList][wordType][word];
														
                                balancer[wordType].push(word);
							
                                notFound = false;
                            }
                        }
                    }
                    else{
				
                        notFound = true;
					
                        while(notFound){						
                            var faceNum  = _CommonAll.randomIndex(2); //get a random face type
                            var pic;
                                
                            // brb: we are missing African-American/15.jpg
                            // so we can't go past 14 for that folder
                            if (dspFaces[faceNum] == "African-American") {
                                pic      = _CommonAll.randomIndex(15);
                            }
                            else {
                                pic      = _CommonAll.randomIndex(16); //and a random pic
                            }
						
                            if($.inArray(pic, balancer[dspFaces[faceNum]]) === -1 && balancer[dspFaces[faceNum]].length < maxFaces){
                                balancer[dspFaces[faceNum]].push(pic);
							
                                tmpFace = "iaimages/folder"+_experimentSettings.faces+"/"+dspFaces[faceNum]+"/"+pic+".jpg";
							
                                notFound = false;
                            }
                        }
                    }
					
                    if(whichDisplay != 'first') {
                        //add the item to the list
                        tmpItem = new listItem("block",
                            reverse,
                            dspWords[0]+"<br/>"+dspFaces[0],
                            dspWords[1]+"<br/>"+dspFaces[1],
                            {
                                type: _experimentData.blocks[i].type,
                                img: tmpFace,
                                face: dspFaces[faceNum],
                                text: tmpWord
                            
                            },
                            null,
                            null,
                            true);
                    }
                    else {
                        
                        tmpItem = new listItem("block",
                            reverse,
                            dspWords[0]+"<br/>"+dspFaces[0],
                            dspWords[1]+"<br/>"+dspFaces[1],
                            {
                                type: _experimentData.blocks[i].type,
                                face: dspFaces[faceNum],
                                text: tmpWord
                            
                            },
                            null,
                            null,
                            true);
                    }
                    _theList.push(tmpItem);
                }
				
                //set them back to normal for the next round
                if(reverse){
                    dspWords.reverse();
                }
            }
        }
		
        function objToArray(obj){
            var tmpList = [];
			 
            for(var i in obj){
                tmpList.push(obj[i]);
            }
			 
            return tmpList;
        }
    }
}
